aboutsummaryrefslogtreecommitdiff
path: root/archival/libunarchive/find_list_entry.c
diff options
context:
space:
mode:
Diffstat (limited to 'archival/libunarchive/find_list_entry.c')
-rw-r--r--archival/libunarchive/find_list_entry.c40
1 files changed, 36 insertions, 4 deletions
diff --git a/archival/libunarchive/find_list_entry.c b/archival/libunarchive/find_list_entry.c
index 57ffec6ec..d1afc72ce 100644
--- a/archival/libunarchive/find_list_entry.c
+++ b/archival/libunarchive/find_list_entry.c
@@ -9,14 +9,46 @@
#include <stdlib.h>
#include "unarchive.h"
/* Find a string in a list */
const llist_t *find_list_entry(const llist_t *list, const char *filename)
{
while (list) {
- if (fnmatch(list->data, filename, FNM_LEADING_DIR) == 0) {
- return (list);
+ if (fnmatch(list->data, filename, 0) == 0) {
+ return list;
}
list = list->link;
}
- return(NULL);
+ return NULL;
+}
+
+/* Same, but compares only path components present in pattern
+ * (extra trailing path components in filename are assumed to match)
+ */
+const llist_t *find_list_entry2(const llist_t *list, const char *filename)
+{
+ char buf[PATH_MAX];
+ int pattern_slash_cnt;
+ const char *c;
+ char *d;
+
+ while (list) {
+ c = list->data;
+ pattern_slash_cnt = 0;
+ while (*c)
+ if (*c++ == '/') pattern_slash_cnt++;
+ c = filename;
+ d = buf;
+ /* paranoia is better that buffer overflows */
+ while (*c && d != buf + sizeof(buf)-1) {
+ if (*c == '/' && --pattern_slash_cnt < 0)
+ break;
+ *d++ = *c++;
+ }
+ *d = '\0';
+ if (fnmatch(list->data, buf, 0) == 0) {
+ return list;
+ }
+ list = list->link;
+ }
+ return NULL;
}
@@ -9,14 +9,46 @@
#include <stdlib.h>
#include "unarchive.h"
-/* Find a string in a list */
+/* Find a string in a shell pattern list */
const llist_t *find_list_entry(const llist_t *list, const char *filename)
{